//+--------------------------------------------------------------------------------------------+
//|                                                                                            |
//|                                 SonicR S&R Lines.mq4                                       | 
//|                                                                                            |
//+--------------------------------------------------------------------------------------------+ 
#property copyright "Copyright @ 2011 traderathome"
#property link      "email: traderathome@msn.com" 

/*---------------------------------------------------------------------------------------------
Overview:

This indicator handles the display of S&R lines, according to the same varied options provided
in the SonicR Chart Panel for the display of horizontal lines.  

There is no known coding (fractal or otherwise) that can pick up "best" locations for S&R line 
placements; there are too many variables involved.  The eye, however, can quickly discern these 
best locations.  Therefore, this indicator does not compute S&R, but relies on the user input  
of the S&R line values directly into the code of this indicator. 

It is suggested that for any symbol traded, you input the S&R values right into the compile of 
this indicator.  Once the inputs are complete for a specific symbol, add that symbol ID to this
indicator name and save it as an additional indicator (i.e., using EU as an example, this indy
SonicR S&R.mq4 is then renamed and saved as SonicR S&R EURUSD).  Now you have a symbol
specific indicator that has the S&R levels stored in it.  You can add the indicator to any 
chart for that symbol at anytime without having to recreate S&R levels!

With any SonicR S&R XXXYYY indicator, to make an indicator for a different forex pair...
 
  1. rename the indicator by changing the symbol in the indicator name to the symbol of the 
     different forex pair
  2. replace the XXXYYY S&R values with S&R values for the different pair (instructions below)
  3. save the new indicator  

Here is where to enter your S&R line data.  Scroll down thru the code to the "int start()"
section.  Just a bit below that is the "//S&R lines-----" section.  Within this section is the
"//S&R Lines data to subroutine.......", where you will enter the S&R line values.  Each line
of code is identical to the others, excepting for the specific S&R line value.  You can easily
add, delete and reorder lines to maintain a numerical sequence, which is not required but will
help you manage your list of S&R lines into the future.  If you are undecided about a specific
level you have created, simply add "//" in front of the line and the code will not display it.
Adding "//" to all lines above and below a limited group of S&R lines limits the display to 
just that group, and keeps the full list of individual S&R levels available.

Display Options-
The S&R lines can be displayed differently, using display numbers 0-4.   

  "0" draws the lines across the chart. 
  "1" draws the lines from chart left thru the current session.  
  "2" draws the lines only thru the current session.  
  "3" draws the lines from the start of the current session to chart right.  
  "4" draws the lines from the current candle to chart right.  
  
This last option allows you to shift the start of the lines further away from the current 
candle using the input "Display_4_Shift_Line_Right".  The feature to start lines further right 
from the current candle begins lines at a specific time in the future based on the number 
(of TFs) you input to "Display_4_Shift_Line_Right".  If you are using a chart with the standard 
chart zoom and you input "46" (TFs) to shift lines to the right, and then you zoom in 1 step, 
the lines may disappear because they start too far into the future to be seen on the wider 
candle chart!  To make the lines start in about the same location relative to the right 
chart border on both charts, a lower number would need to be used for the wider candle chart.  
Notes in the External Inputs provide sample numbers that start lines at an similar distance 
in from the right chart border for the various chart zoom settings.  When a "0" setting is
used for "Display_4_Shift_Line_Right" lines begin at the last candle, and you have a choice
of where to place the labels; either at the beginning of the line, or MaxRight.  To place the
labels at the beginning of the line, make "Line_Labels_MaxRight" = false, and the value for
"MaxRight_Shift_Label_Right" does not matter.  To place the labels MaxRight on the line, make 
"Line_Labels_MaxRight" = true, and input a value for "MaxRight_Shift_Label_Right" the same as
the value used for "Display_4_Shift_Line_Right" when using that mode, which would be "42" for
the normal chart zoom setting.

This indicator can be turned On/Off, preserving your line settings when the lines are not 
being displayed.  You can also select a TF, above which the S&R lines will automatically not
be displayed. 

                                                                    - Traderathome, 07-16-2011                                                                    
------------------------------------------------------------------------------------------------
Suggested Settings:          White Chart              Black Chart 
        
SR_Lines_Color               DarkSlateGray            C'88,88,0' 
SR_Labels_Color              MidnightBlue             C'148,148,148'                     
----------------------------------------------------------------------------------------------*/


//+--------------------------------------------------------------------------------------------+
//| Indicator Global Inputs                                                                    |                                                        
//+--------------------------------------------------------------------------------------------+ 
#property indicator_chart_window

//Global External Inputs------------------------------------------------------------------------ 
extern string Part_1                          = "Indicator Master Controls:";
extern bool   Indicator_On                    = true;
extern string For_Symbol                      = "AUDUSD";
extern int    Display_Max_TF                  = 1440;
extern bool   Use_For_Forex                   = true;
extern bool   Display_5_Digits_Forex          = false;
extern string TF_Choices_1                    = "M1 - H4 =  1, 5, 15, 30, 60, 240";
extern string TF_Choices_2                    = "D  W  M =  1440,  10080,  43200";

extern string __                              = "";
extern string Part_2                          = "Horizontal Lines Position Settings:";
extern string note0                           = "0 = Full screen";
extern string note1                           = "1 = left thru current session";
extern string note2                           = "2 = Current session only";
extern string note3                           = "3 = Current session to right";
extern string note4                           = "4 = Last candle to right (+Shift_Right)";
extern int    SR_Lines_Display_Number         = 0;
extern string note5                           = "Sample equivalent '+Shift_Right'";
extern string note6                           = "numbers for Display Number 4:";
extern string note7                           = "Zoom out 2 = 164";
extern string note8                           = "Zoom out 1 = 82";
extern string note9                           = "Std. chart = 42";
extern string note10                          = "Zoom in 1  = 22";
extern string note11                          = "Zoom in 2  = 12";
extern string note12                          = "Zoom in 3  = 6";
extern int    Display_4_Shift_Line_Right      = 42;

extern string ___                             = "";
extern string Part_3                          = "SR Lines Settings:"; 
extern color  SR_Lines_Color                  = C'88,88,0';
extern int    SR_LineStyle_01234              = 2;    
extern int    SR_SolidLineThickness           = 1;

extern string ____                            = "";
extern string Part_4                          = "SR Labels Settings:";
extern color  SR_Labels_Color                 = C'148,148,148';
extern bool   SR_Labels_Shown                 = true;
extern bool   SR_Labels_Prices                = true;
extern string SR_Labels_FontStyle             = "Verdana"; 
extern int    SR_Labels_FontSize              = 8;  
extern bool   SR_Labels_MaxRight              = false;
extern int    MaxRight_Shift_Label_Right      = 42;  
  
extern string _____                           = "";
extern string Part_5                          = "To add, delete, change S&R, open:";
extern string Note_4                          = "MetaEditor/Navigator/indicators.";
extern string Note_5                          = "Double left click this indicator name.";
extern string Note_6                          = "Follow instructions in the user notes."; 

//Buffers, Constants and Variables-------------------------------------------------------------
bool       FLAG_deinit;

//Deinit Section
int        obj_total,k;
string     name;

//Init Section
int        fxDigits;
double     fxPrice; 
string     sub1,sub2,Price; 

//Draw Lines
int        a,b,c,R2,T4;
int        d1,d2,d3,d4;  
string     line;
datetime   startline, stopline;

//Draw Labels
string     label, spc;
datetime   startlabel; 

//+-------------------------------------------------------------------------------------------+
//| Indicator Initialization                                                                  |                                                        
//+-------------------------------------------------------------------------------------------+      
int init()
   {
   if (Symbol() != For_Symbol) deinit(); return(false);

   //If one, or both, Range Line settings are invalid, exit------------------------------------
   if (SR_Lines_Display_Number <0 || SR_Lines_Display_Number >4)         
      {
      Alert("SonicR S&R: SR Line Number invalid"); return(false);
      }   
   
   FLAG_deinit = false;
  
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator De-initialization                                                               |                                                        
//+-------------------------------------------------------------------------------------------+       
int deinit()
   {   
   obj_total= ObjectsTotal();  
   for (k= obj_total; k>=0; k--)
      {
      name= ObjectName(k); 
      if (StringSubstr(name,0,12)=="[S&R] " + For_Symbol) {ObjectDelete(name);}
      }
   //Comment ("");
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator Start                                                                           |                                                        
//+-------------------------------------------------------------------------------------------+         
int start()
   {   
   //If Indicator is "Off" deinitialize only once, not every tick------------------------------  
   if ((!Indicator_On) || (Symbol() != For_Symbol))
      {
      if (!FLAG_deinit) {deinit(); FLAG_deinit = true;}
      return(0);
      }
            
   //If indicator is "ON", deinitialize and run------------------------------------------------ 
   deinit(); FLAG_deinit = false; 

   //Define fxDigits value to use  
   if (Use_For_Forex)
      {
      sub1=StringSubstr(Symbol(), 3, 3);
      sub2=StringSubstr(Symbol(), 0, 2);
      if (Display_5_Digits_Forex) 
         {
         fxDigits = 5;
         if (sub1 == "JPY") {fxDigits = 3;} 
         if (sub2 == "XA")  {fxDigits = 2;}
         }        
      else 
         {
         fxDigits = 4;
         if (sub1 == "JPY") {fxDigits = 2;}
         if (sub2 == "XA")  {fxDigits = 2;} 
         }         
      }   
  
   //S&R lines----------------------------------------------------------------------
   if (Period()<= Display_Max_TF)
      {
      d1= SR_Lines_Display_Number;
      d2= SR_Lines_Color;
      d3= SR_LineStyle_01234;
      d4= SR_SolidLineThickness;
                              
      //S&R Lines data to subroutine....................................
      //Highlight/copy/paste to duplicate/add lines for new entries
      //Highlight/cut/paste to relocate entries to keep sequential order       
      //Begin line with "//" to not display
      //Highlight/cut to remove a line..................................
                                                                  
      DrawLines("S&R", d1, 1.08843, d2, d3, d4);
      DrawLines("S&R", d1, 1.07752, d2, d3, d4);
      DrawLines("S&R", d1, 1.07416, d2, d3, d4);
    //DrawLines("S&R", d1, 1.07102, d2, d3, d4);
      DrawLines("S&R", d1, 1.07066, d2, d3, d4);
      DrawLines("S&R", d1, 1.06781, d2, d3, d4);
      DrawLines("S&R", d1, 1.06493, d2, d3, d4);
      DrawLines("S&R", d1, 1.06178, d2, d3, d4);
      DrawLines("S&R", d1, 1.05797, d2, d3, d4);
      DrawLines("S&R", d1, 1.05397, d2, d3, d4);
      DrawLines("S&R", d1, 1.05030, d2, d3, d4);
      DrawLines("S&R", d1, 1.04174, d2, d3, d4);
      DrawLines("S&R", d1, 1.03364, d2, d3, d4);
      DrawLines("S&R", d1, 1.02633, d2, d3, d4);
      DrawLines("S&R", d1, 1.01861, d2, d3, d4);    
      }
     
   //End of program computations---------------------------------------------------------------        
   return(0);
   }

//+-------------------------------------------------------------------------------------------+
//| Subroutine To name and draw Daily Pivot and Range HL lines and line labels                |                                                                                  |
//+-------------------------------------------------------------------------------------------+
void DrawLines(string text, int Line_Selection_Number, double level, color Color, int linestyle, int thickness)
   {
   //Lines=====================================================================================  
   //Name lines and labels 
   text = StringTrimLeft(text);       
   line  = "[S&R] " + For_Symbol + " Line " + level;  
   label = "[S&R] " + For_Symbol + " Label " + level;
   
   //Define variables  
   a = linestyle; b = thickness; c =1; if (a==0)c=b;  
   
   //Where to Start lines
   if ((Line_Selection_Number == 0) || (Line_Selection_Number == 1)) {startline = WindowFirstVisibleBar();} //start chart left
   else if(Line_Selection_Number == 2 || Line_Selection_Number == 3) {startline = iTime(NULL, PERIOD_D1, 0);} //start at day separator                  
   else if(Line_Selection_Number == 4) {T4 = Time[0]+ (Period()*60*Display_4_Shift_Line_Right); startline  = T4;} //start right of last candle
                                                                   
   //Where to Stop Lines   
   if(Line_Selection_Number == 0 || Line_Selection_Number == 3) {stopline = Time[0]; R2 = true;}
   else if(Line_Selection_Number == 4) {stopline = T4 + 86400; R2 = true;}   
   else if ((Line_Selection_Number == 1) || (Line_Selection_Number == 2)) {stopline = iTime(NULL,PERIOD_D1,0)+86400; R2 = false;}
                      
   //Draw lines 
   if (ObjectFind(line) != 0)
      {
      ObjectCreate(line, OBJ_TREND, 0, startline, level, stopline, level);    
      ObjectSet(line, OBJPROP_STYLE, a);
      ObjectSet(line, OBJPROP_COLOR, Color);
      ObjectSet(line, OBJPROP_WIDTH, c);
      ObjectSet(line, OBJPROP_BACK, true); 
      ObjectSet(line, OBJPROP_RAY, R2);
      }
   else
      {
      ObjectMove(line, 0, startline, level);
      ObjectMove(line, 1, stopline, level);
      }
  
   //Labels===================================================================================
   //Exit if label not to be shown 
   if (!SR_Labels_Shown) {return (-1);} 
   else 
      {      
      //Align labels depending on if showing prices and on digits in price 
      text = " " + text + "  "; //same initialized spacing as in TzPivots     
      if (SR_Labels_Prices && StrToInteger(text)==0)       
         {
         if (Use_For_Forex) 
            {
            Price=DoubleToStr(level, fxDigits);
            if (Display_5_Digits_Forex) {text = "              " + text + " " + Price;}  //14
            else {text = "            " + text + " " + Price;}  //12            
            }         
         else 
            {
            Price=DoubleToStr(level, Digits);
            spc = "         "; for (k= 1; k<= Digits; k++) {spc = spc + " ";}  //9+
            text = spc + text + " " + Price;
            }
         }   
      else {text = "  " + text + " ";}
                       
      //Where to put labels
      if ((Line_Selection_Number == 0) || (Line_Selection_Number == 1)) //start mid chart                                
         {
         spc = ""; //00 
         startlabel = Time[WindowFirstVisibleBar()/2];
         }             
      else if (Line_Selection_Number == 2 || Line_Selection_Number == 3)
         {            
         if (Time[WindowFirstVisibleBar()] < iTime(NULL, PERIOD_D1, 0)) //start at day separator
            {
            spc="       "; //07                           
            startlabel= iTime(NULL, PERIOD_D1, 0);  
            }          
         else //start start mid chart     
            {
            spc = ""; //00  
            startlabel = Time[WindowFirstVisibleBar()/2];                         
            } 
         }                       
      else if (Line_Selection_Number == 4) //start at current candle   
         {
        
         if (Display_4_Shift_Line_Right > 0)
            {
            spc="         "; //09                   
            startlabel = T4;          
            }         
         else if (SR_Labels_MaxRight)         
            {
            spc="         "; //09                   
            startlabel = Time[0]+ (Period()*60*MaxRight_Shift_Label_Right);             
            } 
         else
            {
            spc="         "; //09                   
            startlabel = Time[0];   
            }       
         }                                     
                                     
      //Draw labels                            
      if (ObjectFind(label) != 0)
         {
         ObjectCreate(label, OBJ_TEXT, 0, startlabel, level);     
         ObjectSetText(label, spc + text, SR_Labels_FontSize, SR_Labels_FontStyle, SR_Labels_Color);
         ObjectSet(label, OBJPROP_BACK, true);
         }        
      else {ObjectMove(label, 0, startlabel, level);}       
      }
   }

//+-------------------------------------------------------------------------------------------+
//| Indicator End                                                                             |                                                        
//+-------------------------------------------------------------------------------------------+      

